/* =============================================================================
 * EMPSE 2008/2009
 *
 * Simple program for ilustrating how to handle keyboard input via a thread
 *   and how to measure time intervals, all of this **WITHOUT BLOKING**
 *
 * Modified: 
 * 2008-10-11 : Created by Paulo Pedreiras (pbrp@ua.pt) 
 *
 * 
 * ============================================================================== */ 
 
/* Includes */ 
#include <stdio.h> 
#include <stdlib.h> 
#include <errno.h> 
#include <sys/time.h> 
#include <unistd.h> 
#include <pthread.h>

/*
 * Some defines ...
 */
#define FALSE 0
#define TRUE  !FALSE
#define INTERVAL 5 

/* 
 * Function/thread prototypes 
 */
void *threadkeyb_function(void *);
int timer_test(int secs);
void timer_set(void);

/*
 * Global vars
 */
char keypress=FALSE, key, end=FALSE, timer_on=FALSE;
struct timeval ts,ta; 

/* 
 *  Main 
 */ 
int main(int argc, char *argv[]) 
{ 
  int res;
  pthread_t threadkeyb;

  /* Create the keyboard handler thread */
  res = pthread_create(&threadkeyb, NULL, threadkeyb_function, NULL);
  if (res != 0) {
    perror("Keyboard thread creation failed");
    return 1;
  }

  printf("\n \"q <ENTER> \" to quit\n\n");

  /* Main loop */
  while(!end) {
    
    /* Process keyboard inputs, in any */
    if(keypress) {
      printf("Key [%c] entered\n",key);
      keypress=FALSE;
    }
    
    /* Generate and detect timer events */
    if(timer_on) {
      if(timer_test(INTERVAL)) {
	printf("Timer event: %d seconds elapsed!\n", INTERVAL);
	timer_on=FALSE;
      }
    }
    else
      timer_set();
    
  }
  
  /* Terminate app */
  res = pthread_join(threadkeyb, NULL);
  if (res != 0) 
    perror("Keyboard handler thread join failed\n");
  
  printf("\nDone\n");  
} 
 

/* 
 * Function / Thread implementation 
 */

/* Check if a given time interval (arg) has elapsed */
int timer_test(int secs)
{
  gettimeofday(&ta, NULL);
  if( (ta.tv_sec -ts.tv_sec) >= secs)
    return TRUE;

  return FALSE;     
}

/* Set the timer_on flag and records start time */
void timer_set(void)
{  gettimeofday(&ts, NULL);
  timer_on=TRUE;
  gettimeofday(&ts, NULL);
}


/* Handles keyboard inputs */
void *threadkeyb_function(void *arg) {
  char cmd[80];

  while(1) {
    scanf("%s",&cmd); // Get one line from terminal
    key=cmd[0];       // Extract first char and signal event
    keypress=TRUE;
    if(key=='q') {    // Terminate thread if appropriate
      end=1;
      break;
    }
  }

  return NULL;
}

